option notes;
%global libroot;

 %let libroot=d:\projects\sugs\mwsug2001\sasdata\final\multi; 

/*%let libroot=c:\users\lhoyle\projects\sugs\mwsug2001\sasdata\final\multi;*/

libname projects "&libroot";

libname samples "&libroot\samples";

                          /* create the logging datasets */

%MACRO INITLOG(sampsiz=100, distrfn=uni);
proc sql;
  create table projects.&distrfn.&sampsiz.rsq
  (distrfn char(4),
   nclusts num,
   sampsiz num,
   rsq num);

   create table work.&distrfn.&sampsiz.cntr
    (distrfn char(4),
     nclusts num,
	 sampsiz num,
	 v num);
quit;

%mend initlog;

          /* macro to run 11 clusterings on one sample  */

%MACRO RUNCLUST(sampsiz=100, distrfn=uni, run=1, dset=foo);

               /* extract one sample from the  dataset of all samples */

data onev;
  keep v;
  start=1 + (&run-1)*&sampsiz;    /* skip run-1 sets of sampsiz observations */
  end=&run*&sampsiz;
  put start= end=;
  do ixp=start to end;
    set samples.&dset point=ixp;
    output;
  end;
  stop;
run;
sasfile onev load;       /*  load the file into memory */ 

      /*   run a fastclus for each of 2 to 12 clusters */
%DO IXF=2 %TO 12;
  proc fastclus data=onev
                maxclusters=&ixf 
                cluster=clust&ixf
                least=2
                                      /* mean=outmn&ixf */
                                      /* out=out&ixf */
                outstat=stat&ixf
                noprint
                ;
        var v;
  run;
                                /*  log the r-squares   */
  proc sql;
    insert into projects.&distrfn.&sampsiz.rsq
         set nclusts=&ixf,
         sampsiz=&sampsiz,
         distrfn="&distrfn",
         rsq=(select v from stat&ixf where _type_='RSQ')
         ;
       
                               /* log the cluster means */
    create table t as
     select "&distrfn" as distrfn,
                 &ixf as nclusts,
                 &sampsiz as sampsiz,
                 v
       from stat&ixf 
       where _type_='CENTER'
       order by v;
  
insert into work.&distrfn.&sampsiz.cntr
	   (distrfn ,
                        nclusts ,
	    sampsiz ,
	    v )
	   select distrfn ,
                       nclusts ,
	    sampsiz ,
	    v 
       from t;
  quit;

%END;

sasfile onev close;      /*  unload  from memory */ 


%MEND RUNCLUST;
                      /*  spin the wheel - a lot    */
       
%MACRO RUNNER(dist=uni, sampsz=100, nruns=50, seed=123841) ;
%let thissamp=s&nruns&dist._&sampsz._&seed;
                        /* create a dataset with all samples in it for this set of runs */
data samples.&thissamp;
  keep s v;
  seed=&seed;
  totaln=&sampsz*&nruns;
  do ix=1 to totaln;
     s=1+floor((ix-1)/&sampsz);
     %IF "&dist" eq "bi" %THEN %DO;
      call rannor(seed,v);
	  call ranuni(seed,u);
	  if u>.5 then v=v+4;
	 %END;
     %ELSE  %IF "&dist" eq "tri" %THEN %DO;
      call rannor(seed,v);
	  call ranuni(seed,u);
	  if u>.3333 then v=v+4;
	  if u>.6667 then v=v+5;
	 %END;
     %ELSE   %DO;
      call ran&dist.(seed,v);
	 %END;
     output;
  end;
run;

%INITLOG(sampsiz=&sampsz, distrfn=&dist);

%DO ixr=1 %TO &nruns;
%RUNCLUST(sampsiz=&sampsz, distrfn=&dist, run=&ixr, dset=&thissamp);
  %put _____________________________run  &ixr  ends;
%END;

     /* renumber the clusters lowest value to highest */
data projects.&&dist.&sampsz.cntr;
  set work.&&dist.&sampsz.cntr;
  retain nc 0;
  retain cluster;
  drop nc;
  if nc ne nclusts then do;
    cluster=0;
  end;
  cluster=cluster+1;
  nc=nclusts;
  output;
run;
%MEND RUNNER;

 
 
/* here is where we specify which samples to run */   
/* capture the time to measure elapsed wall clock */
data _null_;
  start=datetime();
  cstart=put(start,datetime21.3);
  call symput("stmv",cstart);
  put cstart=;
  run;

/*   uncomment the macro invocations to run */
/* or copy one and set fewer runs to test */
    

 
option nonotes;
title "with sasfile";
  
%RUNNER(dist=bi, sampsz=300, nruns=500, seed=5398742);
%RUNNER(dist=tri, sampsz=300, nruns=500, seed=8434685);


%RUNNER(dist=bi, sampsz=1000, nruns=500, seed=7831249);
%RUNNER(dist=tri, sampsz=1000, nruns=500, seed=875961);
%RUNNER(dist=uni, sampsz=1000, nruns=500, seed=293897);
  /*    */
option notes;
                  /* print out the elapsed wall clock */

 data _null_;

 start=input("&stmv",datetime23.);
 now=datetime();
 interval=now-start;
 put "this program took " interval " seconds";
run;



                      /* table all the minimum rsq */

%macro tabmins(sampsiz=1000);
proc sql;
  create table e&sampsiz.min as
  select nclusts, min(rsq) as e&sampsiz.min
  from projects.exp&sampsiz.rsq
   group by nclusts;

  create table n&sampsiz.min as
  select nclusts, min(rsq) as n&sampsiz.min
  from projects.nor&sampsiz.rsq
   group by nclusts;

 create table u&sampsiz.min as
  select nclusts, min(rsq) as u&sampsiz.min
  from projects.uni&sampsiz.rsq
   group by nclusts;



  create table projects.mins&sampsiz as 
  select e&sampsiz.min.nclusts,
         e&sampsiz.min.e&sampsiz.min,
         n&sampsiz.min.n&sampsiz.min,
		 u&sampsiz.min.u&sampsiz.min
  from e&sampsiz.min, n&sampsiz.min, u&sampsiz.min
  where e&sampsiz.min.nclusts=n&sampsiz.min.nclusts and e&sampsiz.min.nclusts=u&sampsiz.min.nclusts;
  quit;

     /* export to an Excel Spreadsheet */
PROC EXPORT DATA= Projects.Mins&sampsiz 
            OUTFILE= "&libroot\mins&sampsiz..xls" 
            DBMS=EXCEL2000 REPLACE;
RUN;

%mend tabmins;

/*
%tabmins(sampsiz=1000);
%tabmins(sampsiz=300);

proc sql;
  create table projects.mins as
  select mins300.nclusts,
         mins300.e300min, mins1000.e1000min,
         mins300.n300min, mins1000.n1000min,
         mins300.u300min, mins1000.u1000min
  from projects.mins1000, projects.mins300
  where mins300.nclusts=mins1000.nclusts;
quit; 


proc means data=PROJECTS.UNI1000RSQ     vardef=DF                                                                                                                
     MIN MAX   MEAN   ;                                                                                                                                                                                                                                                                               
   var rsq ;                                                                                                                                                     
   class nclusts ;                                                                                                                                               
   ;                                                                                                                                                             
run;     

proc means data=PROJECTS.UNI300RSQ     vardef=DF                                                                                                                
     MIN MAX   MEAN   ;                                                                                                                                                                                                                                                                               
   var rsq ;                                                                                                                                                     
   class nclusts ;                                                                                                                                               
   ;                                                                                                                                                             
run;      

                                                                                                                                                                 
                              

*/






            /* scatter plot # clusters by RSQ */

%macro splot(dist=exp, sampsiz=300);
           /* dither the number of clusters */
data &dist.&sampsiz.RSQ;
  set projects.&dist.&sampsiz.RSQ;
  ncfudged=nclusts+(ranuni(0)*.2);
run;

filename spfile "&libroot.\pics\&dist.&sampsiz.RSQ.cgm";

                                  /* cgmmwwc cgmof97p cgmof97l */                                                                                                                               
goptions reset=(axis, legend, pattern, symbol, title, footnote) norotate                                                                                         
         hpos=0 vpos=0 htext=  ctext= target= gaccess=  ;  
 
goptions targetdevice=cgmof97l device=cgmof97L  ctext=blue  ftext='HelveticaBold'
         gsfname=spfile   gsfmode=replace                                                                                                                                  
         graphrc interpol=join;                                                                                                                                
                                                                                 
symbol1 c=DEFAULT i=NONE v=DOT h=1 PCT ;                                                                                                                                              
                                                                                                                                                          
axis1   color=blue width=2.0 style=1                                                                                                                                                          
   label=( height=3 pct  "# of Clusters")                                                                                                                                    
   ;                                                                                                                                                             
axis2   color=blue  width=2.0                                                                                                                                                        
    label=( height=3 pct  " RSQ")  
    order=(0 to 1 by .1) 
  ;                                                                                                                                                             
axis3                                                                                                                                                            
   color=blue                                                                                                                                                    
   width=2.0                                                                                                                                                     
   ;                                                                                                                                                             
                                                                                   
                                                                                                                                                                 
proc gplot data=&dist.&sampsiz.RSQ;                                                                                                                              
   plot rsq * ncfudged /                                                                                                                                          
      haxis=axis1                                                                                                                                                
      vaxis=axis2                                                                                                                                                
      frame                                                                                                                                                      
      ;                                                                                                                                                          
run; quit;                                                                                                                                                       
%mend splot;
                 /* one plot for each combination */

%splot(dist=uni, sampsiz=300);
%splot(dist=uni, sampsiz=1000);

%splot(dist=exp, sampsiz=300);
%splot(dist=exp, sampsiz=1000);

%splot(dist=nor, sampsiz=300);
%splot(dist=nor, sampsiz=1000);

%splot(dist=bi, sampsiz=300);
%splot(dist=bi, sampsiz=1000);

%splot(dist=tri, sampsiz=300);
%splot(dist=tri, sampsiz=1000);


