/*  kmeanMaps.sas   four maps for the MWSUG 2001 paper */
/*                  seven plus or minus 2              */

%let loshade=8;
%let hishade=250;
%global libroot;
%let libroot=d:\projects\sugs\mwsug2001\sasdata\final;

libname ksah 'd:\projects\ksah\sasdata';


proc sql;
  create table pdense as
    select t.county, t.totpop, a.landarea,  t.totpop/a.landarea as pdense
    from (select county, value as totpop from ksah.oneval where varname="TOTPOP" and year=1990 and county ne 0) as t,
         (select county, value as landarea from ksah.oneval where varname="LANDAREA" and year=1990 and county ne 0) as a 
	where t.county=a.county
    order by pdense;
quit;
  
                   /* compute septiles */
data pdense2;
  set pdense;
  septile=ceil(_n_/15);
run;

  proc fastclus data=pdense2
                
                maxclusters=7 
                cluster=clust
                least=2
                out=pdenclus
				mean=pdenmean
                noprint
                ;
	   var pdense;

  run;

proc sql;
  select max(pdense), min(pdense) into :maxden, :minden
  from pdenclus;

  select max(pdense), min(pdense) into :maxmean, :minmean
  from pdenmean;

  create table pdencl2 as
  select pdenclus.*, 
         pdenmean.pdense as pdenmean, 
		 &loshade+(&hishade-&loshade)*(pdenclus.pdense-&minden)/(&maxden-&minden) as shade,
         round(&loshade+(&hishade-&loshade)*(pdenclus.pdense-&minden)/(&maxden-&minden),1) as ishade,

         &loshade+(&hishade-&loshade)*(pdenmean.pdense-&minden)/(&maxden-&minden) as mshade,
         round(&loshade+(&hishade-&loshade)*(pdenmean.pdense-&minden)/(&maxden-&minden),1) as imshade

  from pdenclus, pdenmean
  where pdenclus.clust=pdenmean.clust
  order by pdense;


proc sql;
  create table shades as 
  select distinct ishade, 
      "CX"||put(&hishade-ishade,HEX2.)||put(&hishade-ishade,HEX2.)||"FF" as color
  from pdencl2
  order by ishade;

  create table mshades as 
  select distinct imshade, 
      "CX"||put(&hishade-imshade,HEX2.)||put(&hishade-imshade,HEX2.)||"FF" as mcolor
  from pdencl2
  order by imshade;

  quit;

filename shades TEMP;
data _null_;
       set shades end=last;
       file shades;
       if _n_=1 then put 'goptions colors=(';
       put color ' ';
       if last then put ');';
     run;


filename mshades TEMP;
data _null_;
       set mshades end=last;
       file mshades;
       if _n_=1 then put 'goptions colors=(';
       put mcolor ' ';
       if last then put ');';
     run;


                                  /* cgmmwwc cgmof97p cgmof97l */                                                                                                                               
goptions reset=(axis, legend, pattern, symbol, title, footnote) norotate                                                                                         
         hpos=0 vpos=0 htext= ftext= ctext= target= gaccess=  ;  
 
goptions targetdevice=cgmof97l device=cgmof97L  ctext=blue  
         gsfname=spfile   gsfmode=replace                                                                                                                                  
         graphrc ;      



   %include shades;

filename mapP "&libroot.\pics\MapProportional.cgm";

goptions gsfname=mapP;
goptions
     ctext=black;
   pattern1 v=s;
   pattern2 v=m1x;


title 'Proportional Shading';
  proc gmap                   map=maps.uscounty(where=(state=20)) 
                               data=pdencl2 
                               all;
     id county;
     choro ishade / discrete
            missing
            nolegend
            ctext=black
            cempty=white
            coutline=black    ;
   run;

filename mapCM "&libroot.\pics\MapClusterMeans.cgm";

goptions gsfname=mapCM;

   %include mshades;


title 'Cluster Means';
  proc gmap                   map=maps.uscounty(where=(state=20)) 
                               data=pdencl2 
                               all;
     id county;
     choro pdenmean / discrete
            missing
            nolegend
            ctext=black
            cempty=white
            coutline=black    ;
   run;
   quit;



   %include mshades;

filename mapEI "&libroot.\pics\MapEqualIntervals.cgm";

goptions gsfname=mapEI;
goptions colors=(
CXFAFAFF
CXD1D1FF
CXA9A9FF
CX8080FF
CX5858FF
CX3030FF
CX0707FF);


title 'Equal Intervals';

  proc gmap                   map=maps.uscounty(where=(state=20)) 
                               data=pdencl2 
                               all;
     id county;
     choro pdense / levels=7
	        midpoints=old
            missing
            nolegend
            ctext=black
            cempty=white
            coutline=black    ;
   run;
   quit;



   %include mshades;

filename mapEN "&libroot.\pics\MapEqualNumbers.cgm";

goptions gsfname=mapEN;
goptions colors=(
CXFAFAFF
CXD1D1FF
CXA9A9FF
CX8080FF
CX5858FF
CX3030FF
CX0707FF);

title 'Equal Numbers (septiles)';
  proc gmap                   map=maps.uscounty(where=(state=20)) 
                               data=pdencl2 
                               all;
     id county;
     choro septile / discrete
            missing
            nolegend
            ctext=black
            cempty=white
            coutline=black    ;
   run;
   quit;




quit;

