/*  onesamp   - pull  a single sample from the sample archive */
/*              L. Hoyle   July 2001 */
%let libroot=d:;
libname kmfinal  "&libroot\projects\sugs\mwsug2001\sasdata\final" access=readonly;
libname kmfinsam "&libroot\projects\sugs\mwsug2001\sasdata\final\samples" access=readonly;
/*              these parameters identify a particular sample */

%let sampsiz=300;
%let distrfn=exp;
%let run=243;                    /* run 243 for exp300 123841 is the low r-squared point */
%let nruns=500;
%let seed=123841;
%let thissamp=s&nruns&distrfn._&sampsiz._&seed;
%let graphout=&thissamp._&run..cgm;

%let nclusts=2;

filename boxfile "&libroot\projects\sugs\mwsug2001\sasdata\final\pics\&graphout";

                                /* retrieve one sample   */

data onesamp;
  keep s v;
  start=1 + (&run-1)*&sampsiz;    /* skip run-1 sets of sampsiz observations */
  end=&run*&sampsiz;
  put start= end=;
  do ixp=start to end;
    set kmfinsam.&thissamp point=ixp;
	output;
  end;
  stop;
run;



                                /* rerun the cluster analysis   */
  proc fastclus data=onesamp
                maxclusters=&nclusts 
                cluster=clust&nclusts
                least=2
                mean=outmn&nclusts 
                out=out&nclusts 
                outstat=stat&nclusts
                
                ;
        var v;
  run;


        title;                                                                                                                            
      footnote;                                                                                                                         
title1 "Sample producing R-squared of .2126";                                                                                           
proc sql;                                                                                                                               
   create view WORK._TMP_ as                                                                                                            
      select *,1 as _dummy_                                                                                                             
      from Work.Onesamp                                                                                                                 
      ;                                                                                                                                 
*** Box plot ***;                                                                                                                       
goptions ftext=SWISS ctext=BLACK htext=1 cells; 
goptions targetdevice=cgmof97l device=cgmof97L 
         gsfname=boxfile   gsfmode=replace                                                                                                                                  
         graphrc ;       
symbol1 c=BLACK h=1 cells;                                                                                                              
axis1 major=none value=none label=none;                                                                                                 
proc boxplot data=WORK._TMP_ ;                                                                                                          
   plot (v)*_DUMMY_                                                                                                                   
                            / caxis = BLACK                                                                                             
                              cframe = WHITE                                                                                         
                              ctext = BLACK                                                                                             
                              cboxes = BLACK                                                                                            
                              cboxfill = BLUE                                                                                           
                              idcolor = BLUE                                                                                            
                              idsymbol = DOT                                                                                         
                              boxstyle = SCHEMATIC                                                                                      
                              waxis = 1                                                                                                 
                              name = 'BOX'                                                                                              
                              description = "Box Plot of V"                                                                             
                              npanel = 15                                                                                               
                              haxis = axis1                                                                                             
   ;                                                                                                                                    
run;                                                                                                                                    
symbol1;                                                                                                                                
goptions ftext= ctext= htext=;                                                                                                          
axis1;                                                                                                                                  
proc datasets lib=work nolist;                                                                                                          
     delete _TMP_ /memtype=view;                                                                                                        
run;                                                                                                                                    
   quit;                                                                                                                                
                                             
